@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2019  Sutou Kouhei <kou@cozmixng.org>
+# Copyright (C) 2004-2021  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,6 +43,8 @@ module Rabbit
     def_delegators(:@frame, :iconify, :window)
     def_delegators(:@frame, :fullscreen_available?)
     def_delegators(:@frame, :iconify_available?)
+    def_delegators(:@frame, :toggle_terminal)
+    def_delegators(:@frame, :in_terminal?)
 
     def_delegators(:@renderer, :width, :height)
     # Deprecated. Use #base_width= instead.
@@ -81,6 +83,8 @@ module Rabbit
     def_delegators(:@renderer, :shear_context)
     def_delegators(:@renderer, :save_context, :restore_context)
     def_delegators(:@renderer, :draw_line, :draw_rectangle, :draw_arc)
+    def_delegators(:@renderer, :draw_lines)
+    def_delegators(:@renderer, :draw_polygon)
     def_delegators(:@renderer, :draw_circle, :draw_layout, :draw_pixbuf)
     def_delegators(:@renderer, :draw_arc_by_radius, :draw_circle_by_radius)
     def_delegators(:@renderer, :draw_slide, :draw_polygon, :draw_flag)
@@ -343,7 +347,7 @@ module Rabbit
       apply_theme(@theme_name, &block)
     end
 
-    def parse(source=nil, callback=nil, &block)
+    def parse(source=nil, progress=nil, &block)
       id = Object.new.__id__
       @parse_request_queue.push(id)
       @source = source || @source
@@ -353,7 +357,7 @@ module Rabbit
         keep_index do
           @renderer.pre_parse
           clear
-          Parser.parse(self, @source)
+          Parser.parse(self, @source, progress: progress)
           new_allotted_time = allotted_time
           reset_timer if new_allotted_time != current_allotted_time
           apply_timer
@@ -362,7 +366,7 @@ module Rabbit
             if @parse_request_queue.last != id
               raise ParseFinish
             end
-            callback.call if callback
+            progress.call if progress
           end
           @renderer.post_parse
           index = current_index
@@ -380,9 +384,9 @@ module Rabbit
       end
     end
 
-    def reload_source(callback=nil, &block)
+    def reload_source(progress=nil, &block)
       if need_reload_source?
-        parse(nil, callback, &block)
+        parse(nil, progress, &block)
       end
     end
 
@@ -436,6 +440,14 @@ module Rabbit
       @renderer.post_iconify
     end
 
+    def pre_terminal
+      @renderer.pre_terminal
+    end
+
+    def post_terminal
+      @renderer.post_terminal
+    end
+
     def saved_image_base_name
       if @saved_image_base_name
         base_name = @saved_image_base_name.encode("UTF-8")
